@@ -1,7 +1,5 @@
-#! /bin/sh
-# The next line restarts with tclsh \
-exec /usr/local/bin/tclsh8.5 "$0" ${1+"$@"}
-
+#! /usr/local/bin/tclsh8.5
+## -*- tcl -*-
 # test-histogram.tcl --
 #     Example of a histogram plot with filled background
 #
