@@ -1,6 +1,5 @@
-#! /bin/sh
-# -*- tcl -*- \
-exec /usr/local/bin/tclsh8.5 "$0" ${1+"$@"}
+#! /usr/local/bin/tclsh8.5
+## -*- tcl -*-
 
 package require Tk
 
