@@ -1,7 +1,7 @@
 #==============================================================================
 # Contains some Tk option database settings.
 #
-# Copyright (c) 2004-2013  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -22,22 +22,21 @@ if {[string compare $winSys "x11"] == 0} {
     #
     # Create the font TkDefaultFont if not yet present
     #
-    catch {font create TkDefaultFont -family Helvetica -size -12}
+    catch {font create TkDefaultFont -family Helvetica -size 9}
 
     option add *Font			TkDefaultFont
-    option add *selectBackground	#678db2
+    option add *selectBackground	#5294e2
     option add *selectForeground	white
-} else {
-    option add *ScrollArea.borderWidth			1
-    option add *ScrollArea.relief			sunken
-    option add *ScrollArea.Tablelist.borderWidth	0
-    option add *ScrollArea.Tablelist.highlightThickness	0
-    option add *ScrollArea.Text.borderWidth		0
-    option add *ScrollArea.Text.highlightThickness	0
 }
 option add *Tablelist.background	white
-option add *Tablelist.stripeBackground	#e4e8ec
+option add *Tablelist.stripeBackground	#f0f0f0
 option add *Tablelist.setGrid		yes
 option add *Tablelist.movableColumns	yes
 option add *Tablelist.labelCommand	tablelist::sortByColumn
 option add *Tablelist.labelCommand2	tablelist::addToSortColumns
+option add *ScrollArea.borderWidth			1
+option add *ScrollArea.relief				sunken
+option add *ScrollArea.Tablelist.borderWidth		0
+option add *ScrollArea.Tablelist.highlightThickness	0
+option add *ScrollArea.Text.borderWidth			0
+option add *ScrollArea.Text.highlightThickness		0
