@@ -1,11 +1,9 @@
-#!/bin/sh
-# the next line restarts using wish \
-exec /usr/local/bin/wish8.5 "$0" ${1+"$@"}
+#! /usr/local/bin/wish8.5
 
 #==============================================================================
 # Demo:	wcb::callback <listbox> before selset <callback>
 #
-# Copyright (c) 1999-2010  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 1999-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 package require wcb
@@ -60,6 +58,6 @@ button .close -text Close -command exit
 #
 # Manage the widgets
 #
-pack .spacer .lb -side left -fill y -pady 10
-pack .close -side bottom -padx 10 -pady 10
-pack .picture -padx 10 -pady 10
+pack .spacer .lb -side left -fill y -pady 7p
+pack .close -side bottom -padx 7p -pady 7p
+pack .picture -padx 7p -pady 7p
